<html>
    <head>
        <title>Fitbit Authentication Error</title>
    </head>
    <body>
        <h1>Fitbit Authentication Error</h1>

        <p>We encountered an error while attempting to authenticate you
        through Fitbit.</p>
    </body>
</html>
